@@ -235,6 +235,7 @@ gboolean
 _gnome_vfs_get_current_unix_mounts (GList **return_list)
 {
 	static time_t last_mtime = 0;
+	static off_t last_size = 0;
 	struct mntent *mntent;
 	FILE *file;
 	char *read_file;
@@ -254,11 +255,12 @@ _gnome_vfs_get_current_unix_mounts (GList **return_list)
 		return TRUE;
 	}
 
-	if (sb.st_mtime == last_mtime) {
+	if (sb.st_mtime == last_mtime && sb.st_size == last_size) {
 		return FALSE;
 	}
 
 	last_mtime = sb.st_mtime;
+	last_size = sb.st_size;
 
 	file = setmntent (read_file, "r");
 	if (file == NULL) {
@@ -332,6 +334,7 @@ gboolean
 _gnome_vfs_get_current_unix_mounts (GList **return_list)
 {
 	static time_t last_mtime = 0;
+	static off_t last_size = 0;
 	struct mnttab mntent;
 	FILE *file;
 	char *read_file;
@@ -350,11 +353,12 @@ _gnome_vfs_get_current_unix_mounts (GList **return_list)
 		return TRUE;
 	}
 
-	if (sb.st_mtime == last_mtime) {
+	if (sb.st_mtime == last_mtime && sb.st_size == last_size) {
 		return FALSE;
 	}
 
 	last_mtime = sb.st_mtime;
+	last_size = sb.st_size;
 
 	file = setmntent (read_file, "r");
 	if (file == NULL) {
@@ -512,9 +516,12 @@ _gnome_vfs_get_current_unix_mounts (GList **return_list)
 static char *
 get_fstab_file (void)
 {
-#ifdef _PATH_MNTTAB
+#if defined(HAVE_SYS_MNTCTL_H) && defined(HAVE_SYS_VMOUNT_H) && defined(HAVE_SYS_VFS_H)
+	/* AIX */
+	return "/etc/filesystems";
+#elif defined(_PATH_MNTTAB)
 	return _PATH_MNTTAB;
-#elif defined VFSTAB
+#elif defined(VFSTAB)
 	return VFSTAB;
 #else
 	return "/etc/fstab";
@@ -526,6 +533,7 @@ gboolean
 _gnome_vfs_get_unix_mount_table (GList **return_list)
 {
 	static time_t last_mtime = 0;
+	static off_t last_size = 0;
 	struct mntent *mntent;
 	FILE *file;
 	char *read_file;
@@ -544,11 +552,12 @@ _gnome_vfs_get_unix_mount_table (GList **return_list)
 		return TRUE;
 	}
 
-	if (sb.st_mtime == last_mtime) {
+	if (sb.st_mtime == last_mtime && sb.st_size == last_size) {
 		return FALSE;
 	}
 
 	last_mtime = sb.st_mtime;
+	last_size = sb.st_size;
 
 	file = setmntent (read_file, "r");
 	if (file == NULL) {
@@ -589,6 +598,7 @@ _gnome_vfs_get_unix_mount_table (GList **return_list)
 #ifdef HAVE_HASMNTOPT
 		    || (hasmntopt (mntent, "user") != NULL
 			&& hasmntopt (mntent, "user") != hasmntopt (mntent, "user_xattr"))
+		    || hasmntopt (mntent, "pamconsole") != NULL
 		    || hasmntopt (mntent, "users") != NULL
 		    || hasmntopt (mntent, "owner") != NULL
 #endif
@@ -613,6 +623,7 @@ gboolean
 _gnome_vfs_get_unix_mount_table (GList **return_list)
 {
 	static time_t last_mtime = 0;
+	static off_t last_size = 0;
 	struct mnttab mntent;
 	FILE *file;
 	char *read_file;
@@ -630,11 +641,12 @@ _gnome_vfs_get_unix_mount_table (GList **return_list)
 		return TRUE;
 	}
 
-	if (sb.st_mtime == last_mtime) {
+	if (sb.st_mtime == last_mtime && sb.st_size == last_size) {
 		return FALSE;
 	}
 
 	last_mtime = sb.st_mtime;
+	last_size = sb.st_size;
 
 	file = setmntent (read_file, "r");
 	if (file == NULL) {
@@ -667,6 +679,7 @@ _gnome_vfs_get_unix_mount_table (GList **return_list)
 #ifdef HAVE_HASMNTOPT
 		    || (hasmntopt (&mntent, "user") != NULL
 			&& hasmntopt (&mntent, "user") != hasmntopt (&mntent, "user_xattr"))
+		    || hasmntopt (&mntent, "pamconsole") != NULL
 		    || hasmntopt (&mntent, "users") != NULL
 		    || hasmntopt (&mntent, "owner") != NULL
 #endif
@@ -789,41 +802,34 @@ aix_fs_get (FILE *fd, AixMountTableEntry *prop)
 	return 0;
 }
 
-
-static char *
-get_fstab_file (void)
-{
-	return "/etc/filesystems";
-}
-
-
 gboolean
 _gnome_vfs_get_unix_mount_table (GList **return_list)
 {
 	static time_t last_mtime = 0;
+	static off_t last_size = 0;
 	struct mntent *mntent;
 	FILE *file;
 	char *read_file;
 	char *stat_file;
 	struct stat sb;
 	GnomeVFSUnixMountPoint *mount_entry;
-        MountTableEntry mntent;
+	AixMountTableEntry mntent;
 	
 	stat_file = read_file = get_fstab_file ();
 
 	*return_list = NULL;
 	
-	if (last_mtime != 0) {
-		if (stat (stat_file, &sb) < 0) {
-			g_warning ("Unable to stat %s: %s", stat_file,
-				   g_strerror (errno));
-			return TRUE;
-		}
-		
-		if (sb.st_mtime == last_mtime) {
-			return FALSE;
-		}
+	if (stat (stat_file, &sb) < 0) {
+		g_warning ("Unable to stat %s: %s", stat_file,
+			   g_strerror (errno));
+		return TRUE;
 	}
+		
+	if (last_mtime != 0 && fsb.st_mtime == last_mtime && fsb.st_size == last_size) {
+		return FALSE;
+	}
+	last_mtime = fsb.st_mtime;
+	last_size = fsb.st_size;
 
 	file = setmntent (read_file, "r");
 	if (file == NULL) {
@@ -831,12 +837,13 @@ _gnome_vfs_get_unix_mount_table (GList **return_list)
 	}
 
 	while (!aix_fs_get (file, &mntent)) {
-		if (strcmp ("cdrfs", ent->mnt_fstype) == 0) {
+		if (strcmp ("cdrfs", mntent.mnt_fstype) == 0) {
 			mount_entry = g_new0 (GnomeVFSUnixMountPoint, 1);
 			
-			mount_entry->mount_path = g_strdup (mntent->mnt_dir);
-			mount_entry->device_path = g_strdup (mntent->mnt_fsname);
-			mount_entry->filesystem_type = g_strdup (mntent->mnt_type);
+
+			mount_entry->mount_path = g_strdup (mntent.mnt_mount);
+			mount_entry->device_path = g_strdup (mntent.mnt_special);
+			mount_entry->filesystem_type = g_strdup (mntent.mnt_fstype);
 			mount_entry->is_read_only = TRUE;
 			mount_entry->is_user_mountable = TRUE;
 			
@@ -857,6 +864,7 @@ gboolean
 _gnome_vfs_get_unix_mount_table (GList **return_list)
 {
     	static time_t last_mtime = 0;
+	static off_t last_size = 0;
     	struct fstab *fstab = NULL;
 	char *stat_file;
 	struct stat fsb;
@@ -868,6 +876,8 @@ _gnome_vfs_get_unix_mount_table (GList **return_list)
 #endif
 
 	stat_file = get_fstab_file ();
+
+	*return_list = NULL;
  
     	if (stat (stat_file, &fsb) < 0) {
 	    	g_warning ("Unable to stat %s: %s", stat_file,
@@ -875,10 +885,11 @@ _gnome_vfs_get_unix_mount_table (GList **return_list)
 		return TRUE;
 	}
 
-	if (last_mtime != 0 && fsb.st_mtime == last_mtime) {
+	if (last_mtime != 0 && fsb.st_mtime == last_mtime && fsb.st_size == last_size) {
 	    	return FALSE;
 	}
 	last_mtime = fsb.st_mtime;
+	last_size = fsb.st_size;
  
 	*return_list = NULL;
 
@@ -897,6 +908,14 @@ _gnome_vfs_get_unix_mount_table (GList **return_list)
 		mib[1] = VFS_USERMOUNT;
 		sysctl (mib, 2, &usermnt, &len, NULL, 0);
 	}
+#elif defined(CTL_KERN) && defined(KERN_USERMOUNT)
+	{
+		int mib[2];
+
+		mib[0] = CTL_KERN;
+		mib[1] = KERN_USERMOUNT;
+		sysctl (mib, 2, &usermnt, &len, NULL, 0);
+	}
 #endif
 #endif
 
