@@ -621,7 +621,7 @@ do_read (GnomeVFSMethod *method,
 	if (gzip_handle->last_z_result != Z_OK) {
 		if (gzip_handle->last_z_result == Z_STREAM_END) {
 			*bytes_read = 0;
-			return GNOME_VFS_OK;
+			return GNOME_VFS_ERROR_EOF;
 		} else
 			return result_from_z_result (gzip_handle->last_z_result);
 	} else if (gzip_handle->last_vfs_result != GNOME_VFS_OK) {
@@ -737,7 +737,6 @@ do_get_file_info  (GnomeVFSMethod *method,
 
 		/* guess the mime type of the file inside */
 		/* FIXME bugzilla.eazel.com 2791: guess mime based on contents */
-		g_free(file_info->mime_type);
 		file_info->mime_type = g_strdup(gnome_vfs_mime_type_from_name(file_info->name));
 	}
 
